==> operation-sdmx-1.0.0-20131010.094939-13-sources.jar: org/gcube/data/analysis/tabulardata/operation/sdmx/SDMXCodelistExporter.java <==
package org.gcube.data.analysis.tabulardata.operation.sdmx;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.dbutils.DbUtils;
import org.gcube.data.analysis.tabulardata.cube.CubeManager;
import org.gcube.data.analysis.tabulardata.cube.data.connection.DatabaseConnectionProvider;
import org.gcube.data.analysis.tabulardata.cube.tablemanagers.TableMetaCreator;
import org.gcube.data.analysis.tabulardata.metadata.NoSuchMetadataException;
import org.gcube.data.analysis.tabulardata.model.column.Column;
import org.gcube.data.analysis.tabulardata.model.column.type.AnnotationColumnType;
import org.gcube.data.analysis.tabulardata.model.column.type.CodeColumnType;
import org.gcube.data.analysis.tabulardata.model.column.type.CodeDescriptionColumnType;
import org.gcube.data.analysis.tabulardata.model.column.type.CodeNameColumnType;
import org.gcube.data.analysis.tabulardata.model.metadata.column.DataLocaleMetadata;
import org.gcube.data.analysis.tabulardata.model.metadata.common.LocalizedText;
import org.gcube.data.analysis.tabulardata.model.metadata.common.NamesMetadata;
import org.gcube.data.analysis.tabulardata.model.metadata.table.ExportMetadata;
import org.gcube.data.analysis.tabulardata.model.table.Table;
import org.gcube.data.analysis.tabulardata.operation.worker.BaseWorker;
import org.gcube.data.analysis.tabulardata.operation.worker.OperationInvocation;
import org.gcube.data.analysis.tabulardata.operation.worker.exceptions.OperationException;
import org.gcube.datapublishing.sdmx.api.registry.SDMXRegistryClient;
import org.gcube.datapublishing.sdmx.impl.exceptions.SDMXRegistryClientException;
import org.sdmxsource.sdmx.api.model.mutable.base.AnnotationMutableBean;
import org.sdmxsource.sdmx.api.model.mutable.base.TextTypeWrapperMutableBean;
import org.sdmxsource.sdmx.api.model.mutable.codelist.CodeMutableBean;
import org.sdmxsource.sdmx.api.model.mutable.codelist.CodelistMutableBean;
import org.sdmxsource.sdmx.sdmxbeans.model.mutable.base.AnnotationMutableBeanImpl;
import org.sdmxsource.sdmx.sdmxbeans.model.mutable.base.TextTypeWrapperMutableBeanImpl;
import org.sdmxsource.sdmx.sdmxbeans.model.mutable.codelist.CodeMutableBeanImpl;
import org.sdmxsource.sdmx.sdmxbeans.model.mutable.codelist.CodelistMutableBeanImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.collect.Lists;

public class SDMXCodelistExporter extends BaseWorker {

	private static Logger log = LoggerFactory.getLogger(SDMXCodelistExporter.class);

	private Table table;

	private OperationInvocation invocation;

	private DatabaseConnectionProvider connectionProvider;

	private Column codeColumn;

	private List<Column> nameColumns;

	private List<Column> descriptionColumns;

	private List<Column> annotationColumns;

	private CubeManager cubeManager;

	private String targetUrl;

	private String targetAgency;

	private String targetId;

	private String targetVersion;

	private static String errorMessage = "Unable to complete export procedure";;

	public SDMXCodelistExporter(Table table, OperationInvocation invocation,
			DatabaseConnectionProvider connectionProvider, CubeManager cubeManager) {
		super(invocation);
		this.table = table;
		this.invocation = invocation;
		this.connectionProvider = connectionProvider;
		this.cubeManager = cubeManager;
	}

	@Override
	public void run() {

		try {
			inProgress(0f);
			retrieveNeededColumns();
			retrieveParameters();
			inProgress(0.05f);
			CodelistMutableBean codelist = createBaseCodelistBean();
			inProgress(0.2f);
			populateCodelistWithCodes(codelist);
			inProgress(0.6f);
			publishCodelist(codelist);
			inProgress(0.8f);
			Table resultTable = createMetaTable();
			succeed(resultTable);
		} catch (OperationException e) {
			log.error(errorMessage, e);
			fail(e);
		} catch (Exception e) {
			log.error(errorMessage, e);
			OperationException operationException = new OperationException(errorMessage, e);
			fail(operationException);
		}

	}

	private void retrieveParameters() {
		targetUrl = (String) invocation.getParameterInstances().get(WorkerUtils.REGISTRY_BASE_URL);
		targetAgency = (String) invocation.getParameterInstances().get(WorkerUtils.AGENCY);
		targetId = (String) invocation.getParameterInstances().get(WorkerUtils.ID);
		targetVersion = (String) invocation.getParameterInstances().get(WorkerUtils.VERSION);
	}

	private Table createMetaTable() {
		TableMetaCreator tmc = cubeManager.modifyTableMeta(table.getId());
		tmc.setTableMetadata(new ExportMetadata("SDMX", String.format("%scodelist/%s/%s/%s/", targetUrl, targetAgency,
				targetId, targetVersion), new Date()));
		return tmc.create();
	}

	private void publishCodelist(CodelistMutableBean codelist) throws OperationException {
		String url = (String) invocation.getParameterInstances().get(WorkerUtils.REGISTRY_BASE_URL);
		SDMXRegistryClient registryClient = WorkerUtils.initSDMXClient(url);

		try {
			registryClient.publish(codelist.getImmutableInstance());
		} catch (SDMXRegistryClientException e) {
			throw new OperationException("Unable to publish codelist on registry.", e);
		}
	}

	private void populateCodelistWithCodes(CodelistMutableBean codelist) throws OperationException {
		String query = buildSqlQuery(codeColumn, nameColumns, descriptionColumns, annotationColumns);
		Connection conn = null;
		Statement stmt = null;
		ResultSet rs = null;
		try {
			conn = connectionProvider.getConnection();
			stmt = conn.createStatement();
			rs = stmt.executeQuery(query);

			while (rs.next()) {

				CodeMutableBean code = new CodeMutableBeanImpl();
				code.setId(rs.getString(codeColumn.getName()));

				for (Column column : nameColumns) {
					try {
						Locale columnDataLocale = column.getMetadata(DataLocaleMetadata.class).getLocale();
						code.addName(columnDataLocale.getLanguage(), rs.getString(column.getName()));
					} catch (NoSuchMetadataException e) {
						// This exception should not occur
					}
				}

				for (Column column : descriptionColumns) {
					try {
						Locale columnDataLocale = column.getMetadata(DataLocaleMetadata.class).getLocale();
						code.addDescription(columnDataLocale.getLanguage(), rs.getString(column.getName()));
					} catch (NoSuchMetadataException e) {
						// This exception should not occur
					}
				}

				// TODO-LF: Group annotation columns and create a single one
				for (Column column : annotationColumns) {
					try {
						AnnotationMutableBeanImpl annotation = new AnnotationMutableBeanImpl();
						LocalizedText text = column.getMetadata(NamesMetadata.class).getTextWithLocale(Locale.ENGLISH);
						annotation.setTitle(text.getValue());
						List<TextTypeWrapperMutableBean> annotationValues = Lists.newArrayList();
						annotationValues.add(new TextTypeWrapperMutableBeanImpl(text.getLocaleCode(), rs
								.getString(column.getName())));
						annotation.setText(annotationValues);
						code.addAnnotation(annotation);
					} catch (NoSuchMetadataException e) {
						// This exception should not occur
					}
				}

				codelist.addItem(code);
			}

		} catch (SQLException e) {
			String msg = "Unable to execute database query.";
			log.error(msg, e);
			if (e.getNextException() != null)
				log.error("Inner Exception: ", e.getNextException());
			throw new OperationException(msg, e);
		} finally {
			DbUtils.closeQuietly(rs);
			DbUtils.closeQuietly(stmt);
			DbUtils.closeQuietly(conn);
		}
	}

	private CodelistMutableBean createBaseCodelistBean() {
		CodelistMutableBean codelist = new CodelistMutableBeanImpl();
		codelist.setAgencyId((String) invocation.getParameterInstances().get(WorkerUtils.AGENCY));
		codelist.setVersion((String) invocation.getParameterInstances().get(WorkerUtils.VERSION));
		codelist.setId((String) invocation.getParameterInstances().get(WorkerUtils.ID));

		try {
			List<TextTypeWrapperMutableBean> names = Lists.newArrayList();
			for (LocalizedText text : table.getMetadata(NamesMetadata.class).getTexts()) {
				names.add(new TextTypeWrapperMutableBeanImpl(text.getLocale().getLanguage(), text.getValue()));
			}
			codelist.setNames(names);
		} catch (NoSuchMetadataException e) {
			// TODO-LF: This exception should not occur
		}

		List<AnnotationMutableBean> codelistAnnotations = Lists.newArrayList();
		AnnotationMutableBean sourceAnnotation = new AnnotationMutableBeanImpl();
		sourceAnnotation.setTitle("Source");
		List<TextTypeWrapperMutableBean> textList = Lists.newArrayList();
		textList.add(new TextTypeWrapperMutableBeanImpl("en", "Tabular Data"));
		sourceAnnotation.setText(textList);
		codelistAnnotations.add(sourceAnnotation);
		codelist.setAnnotations(codelistAnnotations);
		return codelist;
	}

	@SuppressWarnings("unchecked")
	private void retrieveNeededColumns() {
		codeColumn = table.getColumnsByType(CodeColumnType.class).get(0);
		nameColumns = table.getColumnsByType(CodeNameColumnType.class);
		descriptionColumns = table.getColumnsByType(CodeDescriptionColumnType.class);
		annotationColumns = table.getColumnsByType(AnnotationColumnType.class);
	}

	private String buildSqlQuery(Column codeColumn, List<Column> nameColumns, List<Column> descriptionColumns,
			List<Column> annotationColumns) {
		StringBuilder sql = new StringBuilder();
		sql.append("SELECT " + codeColumn.getName() + ", ");
		for (Column column : nameColumns) {
			sql.append(column.getName() + " , ");
		}
		for (Column column : descriptionColumns) {
			sql.append(column.getName() + " , ");
		}
		for (Column column : annotationColumns) {
			sql.append(column.getName() + " , ");
		}
		sql.delete(sql.length() - 2, sql.length()); // Delete comma
		sql.append(" FROM " + table.getName() + ";");
		return sql.toString();
	}

}

==> operation-sdmx-2.0.3-3.1.1-sources.jar: org/gcube/data/analysis/tabulardata/operation/sdmx/SDMXCodelistExporter.java <==
package org.gcube.data.analysis.tabulardata.operation.sdmx;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.List;

import org.apache.commons.dbutils.DbUtils;
import org.gcube.data.analysis.tabulardata.cube.CubeManager;
import org.gcube.data.analysis.tabulardata.cube.data.connection.DatabaseConnectionProvider;
import org.gcube.data.analysis.tabulardata.cube.tablemanagers.TableMetaCreator;
import org.gcube.data.analysis.tabulardata.metadata.NoSuchMetadataException;
import org.gcube.data.analysis.tabulardata.model.column.Column;
import org.gcube.data.analysis.tabulardata.model.column.type.AnnotationColumnType;
import org.gcube.data.analysis.tabulardata.model.column.type.CodeColumnType;
import org.gcube.data.analysis.tabulardata.model.column.type.CodeDescriptionColumnType;
import org.gcube.data.analysis.tabulardata.model.column.type.CodeNameColumnType;
import org.gcube.data.analysis.tabulardata.model.metadata.column.DataLocaleMetadata;
import org.gcube.data.analysis.tabulardata.model.metadata.common.LocalizedText;
import org.gcube.data.analysis.tabulardata.model.metadata.common.NamesMetadata;
import org.gcube.data.analysis.tabulardata.model.metadata.table.ExportMetadata;
import org.gcube.data.analysis.tabulardata.model.table.Table;
import org.gcube.data.analysis.tabulardata.operation.invocation.OperationInvocation;
import org.gcube.data.analysis.tabulardata.operation.worker.ImmutableWorkerResult;
import org.gcube.data.analysis.tabulardata.operation.worker.Worker;
import org.gcube.data.analysis.tabulardata.operation.worker.WorkerResult;
import org.gcube.data.analysis.tabulardata.operation.worker.exceptions.WorkerException;
import org.gcube.datapublishing.sdmx.api.registry.SDMXRegistryClient;
import org.gcube.datapublishing.sdmx.impl.exceptions.SDMXRegistryClientException;
import org.sdmxsource.sdmx.api.model.mutable.base.AnnotationMutableBean;
import org.sdmxsource.sdmx.api.model.mutable.base.TextTypeWrapperMutableBean;
import org.sdmxsource.sdmx.api.model.mutable.codelist.CodeMutableBean;
import org.sdmxsource.sdmx.api.model.mutable.codelist.CodelistMutableBean;
import org.sdmxsource.sdmx.sdmxbeans.model.mutable.base.AnnotationMutableBeanImpl;
import org.sdmxsource.sdmx.sdmxbeans.model.mutable.base.TextTypeWrapperMutableBeanImpl;
import org.sdmxsource.sdmx.sdmxbeans.model.mutable.codelist.CodeMutableBeanImpl;
import org.sdmxsource.sdmx.sdmxbeans.model.mutable.codelist.CodelistMutableBeanImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.collect.Lists;

public class SDMXCodelistExporter extends Worker {

	private static Logger log = LoggerFactory.getLogger(SDMXCodelistExporter.class);

	private Table table;

	private OperationInvocation invocation;

	private DatabaseConnectionProvider connectionProvider;

	private Column codeColumn;

	private List<Column> nameColumns;

	private List<Column> descriptionColumns;

	private List<Column> annotationColumns;

	private CubeManager cubeManager;

	private String targetUrl;

	private String targetAgency;

	private String targetId;

	private String targetVersion;

	private static String errorMessage = "Unable to complete export procedure";;

	public SDMXCodelistExporter(Table table, OperationInvocation invocation,
			DatabaseConnectionProvider connectionProvider, CubeManager cubeManager) {
		super(invocation);
		this.table = table;
		this.invocation = invocation;
		this.connectionProvider = connectionProvider;
		this.cubeManager = cubeManager;
	}
	
	@Override
	protected WorkerResult execute() throws WorkerException {
		try {
			retrieveNeededColumns();
			retrieveParameters();
			updateProgress(0.1f);
			CodelistMutableBean codelist = createBaseCodelistBean();
			updateProgress(0.2f);
			populateCodelistWithCodes(codelist);
			updateProgress(0.6f);
			publishCodelist(codelist);
			updateProgress(0.8f);
			Table resultTable = createMetaTable();
			return new ImmutableWorkerResult(resultTable);
		} catch (RuntimeException e) {
			log.error(errorMessage, e);
			throw new WorkerException(errorMessage, e);
		}
	}

	private void retrieveParameters() {
		targetUrl = (String) invocation.getParameterInstances().get(WorkerUtils.REGISTRY_BASE_URL);
		targetAgency = (String) invocation.getParameterInstances().get(WorkerUtils.AGENCY);
		targetId = (String) invocation.getParameterInstances().get(WorkerUtils.ID);
		targetVersion = (String) invocation.getParameterInstances().get(WorkerUtils.VERSION);
	}

	private Table createMetaTable() {
		TableMetaCreator tmc = cubeManager.modifyTableMeta(table.getId());
		tmc.setTableMetadata(new ExportMetadata("SDMX", String.format("%scodelist/%s/%s/%s/", targetUrl, targetAgency,
				targetId, targetVersion), new Date()));
		return tmc.create();
	}

	private void publishCodelist(CodelistMutableBean codelist) throws WorkerException {
		String url = (String) invocation.getParameterInstances().get(WorkerUtils.REGISTRY_BASE_URL);
		SDMXRegistryClient registryClient = WorkerUtils.initSDMXClient(url);

		try {
			registryClient.publish(codelist.getImmutableInstance());
		} catch (SDMXRegistryClientException e) {
			throw new WorkerException("Unable to publish codelist on registry.", e);
		}
	}

	private void populateCodelistWithCodes(CodelistMutableBean codelist) throws WorkerException {
		String query = buildSqlQuery(codeColumn, nameColumns, descriptionColumns, annotationColumns);
		Connection conn = null;
		Statement stmt = null;
		ResultSet rs = null;
		try {
			conn = connectionProvider.getConnection();
			stmt = conn.createStatement();
			rs = stmt.executeQuery(query);

			while (rs.next()) {

				CodeMutableBean code = new CodeMutableBeanImpl();
				code.setId(rs.getString(codeColumn.getName()));

				for (Column column : nameColumns) {
					try {
						String columnDataLocale = column.getMetadata(DataLocaleMetadata.class).getLocale();
						code.addName(columnDataLocale, rs.getString(column.getName()));
					} catch (NoSuchMetadataException e) {
						// This exception should not occur
					}
				}

				for (Column column : descriptionColumns) {
					try {
						String columnDataLocale = column.getMetadata(DataLocaleMetadata.class).getLocale();
						code.addDescription(columnDataLocale, rs.getString(column.getName()));
					} catch (NoSuchMetadataException e) {
						// This exception should not occur
					}
				}

				// TODO-LF: Group annotation columns and create a single one
				for (Column column : annotationColumns) {
					try {
						AnnotationMutableBeanImpl annotation = new AnnotationMutableBeanImpl();
						LocalizedText text = column.getMetadata(NamesMetadata.class).getTextWithLocale("en");
						annotation.setTitle(text.getValue());
						List<TextTypeWrapperMutableBean> annotationValues = Lists.newArrayList();
						annotationValues.add(new TextTypeWrapperMutableBeanImpl(text.getLocale(), rs
								.getString(column.getName())));
						annotation.setText(annotationValues);
						code.addAnnotation(annotation);
					} catch (NoSuchMetadataException e) {
						// This exception should not occur
					}
				}
				codelist.addItem(code);
			}

		} catch (SQLException e) {
			String msg = "Unable to execute database query.";
			log.error(msg, e);
			if (e.getNextException() != null)
				log.error("Inner Exception: ", e.getNextException());
			throw new WorkerException(msg, e);
		} finally {
			DbUtils.closeQuietly(rs);
			DbUtils.closeQuietly(stmt);
			DbUtils.closeQuietly(conn);
		}
	}

	private CodelistMutableBean createBaseCodelistBean() {
		CodelistMutableBean codelist = new CodelistMutableBeanImpl();
		codelist.setAgencyId((String) invocation.getParameterInstances().get(WorkerUtils.AGENCY));
		codelist.setVersion((String) invocation.getParameterInstances().get(WorkerUtils.VERSION));
		codelist.setId((String) invocation.getParameterInstances().get(WorkerUtils.ID));

		try {
			List<TextTypeWrapperMutableBean> names = Lists.newArrayList();
			for (LocalizedText text : table.getMetadata(NamesMetadata.class).getTexts()) {
				names.add(new TextTypeWrapperMutableBeanImpl(text.getLocale(), text.getValue()));
			}
			codelist.setNames(names);
		} catch (NoSuchMetadataException e) {
			// TODO-LF: This exception should not occur
		}

		List<AnnotationMutableBean> codelistAnnotations = Lists.newArrayList();
		AnnotationMutableBean sourceAnnotation = new AnnotationMutableBeanImpl();
		sourceAnnotation.setTitle("Source");
		List<TextTypeWrapperMutableBean> textList = Lists.newArrayList();
		textList.add(new TextTypeWrapperMutableBeanImpl("en", "Tabular Data"));
		sourceAnnotation.setText(textList);
		codelistAnnotations.add(sourceAnnotation);
		codelist.setAnnotations(codelistAnnotations);
		return codelist;
	}

	@SuppressWarnings("unchecked")
	private void retrieveNeededColumns() {
		codeColumn = table.getColumnsByType(CodeColumnType.class).get(0);
		nameColumns = table.getColumnsByType(CodeNameColumnType.class);
		descriptionColumns = table.getColumnsByType(CodeDescriptionColumnType.class);
		annotationColumns = table.getColumnsByType(AnnotationColumnType.class);
	}

	private String buildSqlQuery(Column codeColumn, List<Column> nameColumns, List<Column> descriptionColumns,
			List<Column> annotationColumns) {
		StringBuilder sql = new StringBuilder();
		sql.append("SELECT " + codeColumn.getName() + ", ");
		for (Column column : nameColumns) {
			sql.append(column.getName() + " , ");
		}
		for (Column column : descriptionColumns) {
			sql.append(column.getName() + " , ");
		}
		for (Column column : annotationColumns) {
			sql.append(column.getName() + " , ");
		}
		sql.delete(sql.length() - 2, sql.length()); // Delete comma
		sql.append(" FROM " + table.getName() + ";");
		return sql.toString();
	}

}
